@@ -34,11 +34,11 @@ scrolling, border setting, and more in development.
 package Term::Graille;
 
 use strict;use warnings;
-our $VERSION="0.09";
+our $VERSION="0.10";
 use utf8;
 use open ":std", ":encoding(UTF-8)";
 use base 'Exporter';
-our @EXPORT_OK = qw/colour paint printAt clearScreen border blockBlit block2braille pixelAt cursorAt/;
+our @EXPORT_OK = qw/colour paint printAt clearScreen border blockBlit block2braille pixelAt cursorAt wrapText/;
 use Algorithm::Line::Bresenham 0.15;
 use Time::HiRes "sleep";
 
@@ -369,9 +369,10 @@ sub scroll{
 		/^r/i && do{
 			foreach my $row (0..$#{$self->{grid}}){
 				my @r=@{$self->{grid}->[$row]};
-				my $end=pop(@r);
-				unshift(@r,$wrap?$end:'⠀');
-				$self->{grid}->[$row]=\@r;
+				my @end=splice @r, -$numberOfChar, $numberOfChar;
+				@end=('⠀')x$numberOfChar unless $wrap;
+			#	unshift(@r,$wrap?@end:(('⠀')x$numberOfChar));
+				$self->{grid}->[$row]=[@end,@r];
 			}
 			last;
 		};
@@ -379,26 +380,29 @@ sub scroll{
 		/^l/i && do{
 			foreach my $row (0..$#{$self->{grid}}){
 				my @r=@{$self->{grid}->[$row]};
-				my $end=shift(@r);
-				push(@r,$wrap?$end:'⠀');
-				$self->{grid}->[$row]=\@r;
+				my @end=splice @r, 0, $numberOfChar;
+				@end=('⠀')x$numberOfChar unless $wrap;
+			#	push(@r,$wrap?@end:(('⠀')x$numberOfChar));
+				$self->{grid}->[$row]=[@r,@end];
 			}
 			last;
 		};
 				
 		/^d/i && do{
 				my @r=@{$self->{grid}};
-				my $end=shift(@r);
-				push(@r,$wrap?$end:[('⠀')x ($self->{width}/2+($self->{width}%2?1:0))]);
-				$self->{grid}=\@r;
+				my @end=splice @r, 0, $numberOfChar;
+				@end=([('⠀')x ($self->{width}/2+($self->{width}%2?1:0))])x$numberOfChar unless $wrap;
+			#	push(@r,$wrap?@end:(([('⠀')x ($self->{width}/2+($self->{width}%2?1:0))])x$numberOfChar));
+				$self->{grid}=[@r,@end];
 			last;
 		};
 				
 		/^u/i && do{
 				my @r=@{$self->{grid}};
-				my $end=pop(@r);
-				unshift(@r,$wrap?$end:[('⠀')x ($self->{width}/2+($self->{width}%2?1:0))]);
-				$self->{grid}=\@r;
+				my @end=splice @r, -$numberOfChar, $numberOfChar;
+				@end=([('⠀')x ($self->{width}/2+($self->{width}%2?1:0))])x$numberOfChar unless $wrap;
+			#	unshift(@r,$wrap?@end:(([('⠀')x ($self->{width}/2+($self->{width}%2?1:0))])x$numberOfChar));
+				$self->{grid}=[@end,@r];
 			last;
 		};		
 	}
@@ -468,6 +472,7 @@ othe $canvas drawing actions.  The optional C<$fmt> allows the setting of colour
 
 sub textAt{
 	my ($self,$x,$y,$text,$fmt)=@_;
+	return unless defined $text;
 	my ($chrX,$chrY,$xOffset,$yOffset)=charOffset($self,$x,$y);
 	if ($chrX!=-1){
 		my @chrs=split(//,$text);
@@ -539,6 +544,7 @@ sub logo{
 	my @commands= map{s/^\s+|\s+$//g; $_}split(/[\n;]+/,$script);
 	foreach my $instr (@commands){
 		next unless $instr;
+		$instr=~s/#\.*$//;
 		next if ($instr=~/#/);
 		my ($c,$p)=split(/[\s]+/,$instr,2);
 		my @pars=split(/,/,$p) if $p;
@@ -561,7 +567,7 @@ sub logo{
 			last;
 		};
 		/^(rt|right)/  && do{
-			last unless ($pars[0] && (0+$pars[0]));
+			last unless ($pars[0] && ($pars[0]=~/^\d+$/));
 			$self->{logoVars}->{d}-=$pars[0];
 			$self->{logoVars}->{d}+=360 while($self->{logoVars}->{d}<360);
 			last;
@@ -597,6 +603,10 @@ sub logo{
 			$self->{logoVars}->{y}=$self->{height}/2;
 			last;
 		};
+		/^cs/  && do{
+			$self->clear();
+			last;
+		};
 		/^sp/ && do{
 			$self->{logoVars}->{sp}=$pars[0];
 			last
@@ -664,7 +674,8 @@ our %borders=(
 
 our %colours=(black   =>30,red   =>31,green   =>32,yellow   =>33,blue   =>34,magenta   =>35,cyan  =>36,white   =>37,
                on_black=>40,on_red=>41,on_green=>42,on_yellow=>43,on_blue=>44,on_magenta=>4,on_cyan=>46,on_white=>47,
-               reset=>0, bold=>1, italic=>3, underline=>4, strikethrough=>9,);
+               reset=>0, bold=>1, italic=>3, underline=>4, blink=>5, strikethrough=>9, invert=>7,);
+               
 sub printAt{
   my ($row,$column,@textRows)=@_;
   @textRows = @{$textRows[0]} if ref $textRows[0];  
@@ -677,10 +688,8 @@ sub printAt{
 sub cursorAt{
 	my ($r,$c)=@_;
 	return "\033[?25l\033[".$r.";".$c."H";
-	
 }
 
-
 sub border{
 	my ($top,$left,$bottom,$right,$style,$colour,$title,$titleColour)=@_;
 	$style//="simple";
@@ -715,6 +724,33 @@ sub colour{
   return join "",map {defined $colours{$_}?"\033[$colours{$_}m":""} @formats;
 }
 
+sub wrapText{
+	my ($str,$width)=@_;
+	my @lines=();
+	my $line="";
+	$str=~s/ +/ /gm;
+	$str=~s/\n/ \n /gm;
+	foreach my $word(split / /,$str){
+		if ($word eq "\n"){
+			push @lines,$line;
+			$line="";
+		}
+		elsif (1+length $line.$word > $width){
+			push @lines,$line;
+			$line=$word;
+		}
+		elsif ($line eq "") {
+			$line=$word
+		}
+		else{
+			$line=$line." ".$word
+		}
+	}
+	push @lines,$line;
+	return \@lines;
+}
+
+
 
 # given an 8 bit block of data, produce a braille block
 sub block2braille{
